var IMWIDTH = 750;
var IMHEIGHT = 140;
var IMBASEY = 0;
var X0 = IMWIDTH / 2;
var Y0 = IMBASEY + IMHEIGHT / 2;
var JER_PX = 13; // Jupiter equatorial radius
var JPR_PX = 11; // Jupiter polar radius
var JUPITER_COLOUR = "#ddd";
var MOON_RADIUS_PX = 2;
var MOON_DIAMETER_PX = MOON_RADIUS_PX * 2;
var MOON_COLOURS = ["#f00", "#ff0", "#3f3", "#36f"];
var BACKGROUND = "#000";
var FONT = "bold 12px sans-serif";

var flippedNS = false;
var flippedEW = false;

function invert ()
{
  flippedNS = !flippedNS;
  flippedEW = !flippedEW;
  showMoons ();
}

function flip ()
{
  flippedNS = !flippedNS;
  showMoons ();
}

function reset ()
{
  flippedNS = false;
  flippedEW = false;
  showMoons ();
}

var PI2 = Math.PI * 2;
var DEGS_TO_RADS = Math.PI / 180.0;
var RADS_TO_DEGS = 180.0 / Math.PI;
function acosDegs (x) { return RADS_TO_DEGS * Math.acos (x); }
function asinDegs (x) { return RADS_TO_DEGS * Math.asin (x); }
function cos (xDegs) { return Math.cos (xDegs * DEGS_TO_RADS); }
function sin (xDegs) { return Math.sin (xDegs * DEGS_TO_RADS); }
function in360 (xDegs) { return xDegs % 360.0; }

var cal = null;

function setCal (theCalendar)
{
  cal = theCalendar;
  showMoons ();
}

function showMoons ()
{
  if (!!document.createElement ("canvas").getContext)
  {
    var canvas = document.getElementById ("jupiter");
    var g2 = canvas.getContext ("2d");
    g2.fillStyle = BACKGROUND;
    g2.fillRect (0, 0, IMWIDTH, IMHEIGHT + IMBASEY);
    g2.textBaseline = "top";
    g2.font = FONT;

    if (null != cal)
    {
      var jde = cal.getJD ();
      var daysSince2000Jan1_5 = jde - 2451545.0; // Since 2000 Jan 1.5
      var _Vdegs = in360 (172.74 + 0.00111588 * daysSince2000Jan1_5);
      var _329sinV = 0.329 * sin (_Vdegs);
      var _Mdegs = in360 (357.529 + 0.9856003 * daysSince2000Jan1_5);
      var _Ndegs = in360 (20.020 + 0.0830853 * daysSince2000Jan1_5 + _329sinV);
      var _Jdegs = in360 (66.115 + 0.9025179 * daysSince2000Jan1_5 - _329sinV);
      var _Adegs = 1.915 * sin (_Mdegs) + 0.020 * sin (2.0 * _Mdegs);
      var _Bdegs = 5.555 * sin (_Ndegs) + 0.168 * sin (2.0 * _Ndegs);
      var _Kdegs = _Jdegs + _Adegs - _Bdegs;
      var _RAU = 1.00014 - 0.01671 * cos (_Mdegs) - 0.00014 * cos (2.0 * _Mdegs);
      var _rAU = 5.20872 - 0.25208 * cos (_Ndegs) - 0.00611 * cos (2.0 * _Ndegs);
      var deltaAU = Math.sqrt (
        _rAU * _rAU + _RAU * _RAU - 2.0 * _rAU * _RAU * cos (_Kdegs));
      var psiDegs = asinDegs (_RAU * sin (_Kdegs) / deltaAU); // Range -12 .. +12
      var _173 = daysSince2000Jan1_5 - deltaAU / 173.0;
      var psiBdegs = psiDegs - _Bdegs;
      var lambdaDegs = 34.35 + 0.083091 * daysSince2000Jan1_5 + _329sinV + _Bdegs;
      var _DSdegs = 3.12 * sin (lambdaDegs + 42.8);
      var _DEdegs = _DSdegs - 2.22 * sin (psiDegs) * cos (lambdaDegs + 22.0) -
                        1.3 * (_rAU - deltaAU) / deltaAU * sin (lambdaDegs - 100.5);
      var u1degs = in360 (163.8069 + 203.4058646 * _173 + psiBdegs);
      var u2degs = in360 (358.4140 + 101.2916335 * _173 + psiBdegs);
      var u3degs = in360 (5.7176 + 50.2345180 * _173 + psiBdegs);
      var u4degs = in360 (224.8092 + 21.4879800 * _173 + psiBdegs);
      var _Gdegs = 331.18 + 50.310482 * _173;
      var _Hdegs = 87.45 + 21.569231 * _173;
      // In the following, units jer = Jupiter equatorial radius
      var r1jer = 5.9057 - 0.0244 * cos (2.0 * (u1degs - u2degs));
      var r2jer = 9.3966 - 0.0882 * cos (2.0 * (u2degs - u3degs));
      var r3jer = 14.9883 - 0.0216 * cos (_Gdegs);
      var r4jer = 26.3627 - 0.1939 * cos (_Hdegs);
      u1degs += 0.473 * sin (2.0 * (u1degs - u2degs));
      u2degs += 1.065 * sin (2.0 * (u2degs - u3degs));
      u3degs += 0.165 * sin (_Gdegs);
      u4degs += 0.843 * sin (_Hdegs);
      var sinDE = sin (_DEdegs);

      g2.fillStyle = JUPITER_COLOUR;
      g2.save ();
      var scale = JPR_PX / JER_PX;
      g2.scale (1, scale);
      g2.beginPath ();
      g2.arc (X0, Y0 / scale, JER_PX, 0, PI2, false);
      g2.closePath ();
      g2.restore ();
      g2.fill ();

      var _Xjer = new Array (4);
      var _Yjer = new Array (4);
      _Xjer [0] = r1jer * sin (u1degs);
      _Yjer [0] = -r1jer * cos (u1degs) * sinDE;
      _Xjer [1] = r2jer * sin (u2degs);
      _Yjer [1] = -r2jer * cos (u2degs) * sinDE;
      _Xjer [2] = r3jer * sin (u3degs);
      _Yjer [2] = -r3jer * cos (u3degs) * sinDE;
      _Xjer [3] = r4jer * sin (u4degs);
      _Yjer [3] = -r4jer * cos (u4degs) * sinDE;
	  
      var nameY = 4;
      g2.fillStyle = MOON_COLOURS [0];
      g2.fillText ("I Io", 160, nameY);
      g2.fillStyle = MOON_COLOURS [1];
      g2.fillText ("II Europa", 260, nameY);
      g2.fillStyle = MOON_COLOURS [2];
      g2.fillText ("III Ganymede", 410, nameY);
      g2.fillStyle = MOON_COLOURS [3];
      g2.fillText ("IV Callisto", 560, nameY);

      for (i = 0; i < 4; i++)
      {
		if (flippedEW)
		{
		  _Xjer [i] = - _Xjer [i];
		}

		if (flippedNS)
		{
		  _Yjer [i] = - _Yjer [i];
		}

        var xPx = Math.round (X0 + _Xjer [i] * JER_PX);
        var yPx = Math.round (Y0 - _Yjer [i] * JER_PX);
        g2.fillStyle = MOON_COLOURS [i];
    		g2.beginPath ();
        g2.arc (xPx, yPx, MOON_RADIUS_PX, 0, PI2, false);
      	g2.closePath ();
        g2.fill();
      }

      g2.fillStyle = JUPITER_COLOUR;
	  
	  if (flippedEW)
	  {
		g2.fillText ("W", 4, Y0 - 8);
		g2.fillText ("E", IMWIDTH - 16, Y0 - 8);
	  }
	  else
	  {
		g2.fillText ("E", 4, Y0 - 8);
		g2.fillText ("W", IMWIDTH - 16, Y0 - 8);
	  }
	  
	  if (flippedNS)
	  {
		g2.fillText ("S", X0 - 4, IMBASEY + 4);
		g2.fillText ("N", X0 - 4, IMBASEY + IMHEIGHT - 16);
	  }
	  else
	  {
		g2.fillText ("N", X0 - 4, IMBASEY + 4);
		g2.fillText ("S", X0 - 4, IMBASEY + IMHEIGHT - 16);
	  }
	  
      cal.setDateCaption ();
    }
  }
  else
  {
    alert ("Error: could not get context in showMoons()");
  }
}
