/* telecalc.js www.grelf.net 23/2/11 */

var foundErrors = false;
var RAD_TO_DEG = 180 / Math.PI;
var RAD_TO_MIN = RAD_TO_DEG * 60;
var RAD_TO_SEC = RAD_TO_MIN * 60;
var aUnits, lUnits, tUnits, iUnits;
var aW, aH, aT;

function fieldOfView ()
{
	foundErrors = false;
	var form_calc = document.forms ["form_calc"];
	var f = getPositiveFloatField (form_calc.inp_flength.value, "lab_flength");
	var el_lu = form_calc.sel_lUnits;
	lUnits = el_lu.options [el_lu.selectedIndex].value;
	var el_x = form_calc.sel_detector;
	var x = el_x.options [el_x.selectedIndex].value;
	var y = getPositiveFloatField (form_calc.inp_other.value, "lab_other");
	var el_au = form_calc.sel_aUnits;
	aUnits = el_au.options [el_au.selectedIndex].value;
	var el_res = document.getElementById ("results_fov");
	
	if (foundErrors)
	{
		el_res.innerHTML = "&nbsp;";
	}
	else
	{
		// Convert f to mm:
		switch (lUnits)
		{
		case "cm": f *= 10; break;
		case "in": f *= 25.4; break;
		}
		
		// Get angles in radians:
		aW = 2 * Math.atan2 (18, x * y * f);
		aH = 2 * Math.atan2 (12, x * y * f);
		
		// Convert to angle units:
		switch (aUnits)
		{
		case "degrees": aW *= RAD_TO_DEG; aH *= RAD_TO_DEG; break;
		case "arcmin": aW *= RAD_TO_MIN; aH *= RAD_TO_MIN; break;
		case "arcsec": aW *= RAD_TO_SEC; aH *= RAD_TO_SEC; break;
		}
		
		el_res.innerHTML = "Field of view = " + aW.toPrecision (3) + " x " + aH.toPrecision (3) + " " + aUnits;
	}
}

function trail ()
{
	fieldOfView ();

	foundErrors = false;
	var form_calc = document.forms ["form_calc"];
	var t = getPositiveFloatField (form_calc.inp_exp.value, "lab_exp");
	var el_tu = form_calc.sel_tUnits;
	tUnits = el_tu.options [el_tu.selectedIndex].value;
	var dec = getPositiveFloatFieldWithLimits (form_calc.inp_dec.value, "lab_dec", -90, 90);
	var el_res = document.getElementById ("results_trail");
	var wd = getPositiveFloatField (form_calc.inp_wd.value, "lab_wd");
	var ht = getPositiveFloatField (form_calc.inp_ht.value, "lab_ht");
	var el_resim = document.getElementById ("results_image");
	
	if (foundErrors)
	{
		el_res.innerHTML = "&nbsp;";
		el_resim.innerHTML = "&nbsp;";
	}
	else
	{
		// Convert t to seconds:
		switch (tUnits)
		{
		case "ms": t /= 1000; break;
		case "min": t *= 60; break;
		}
		
		var trailDegs = Math.cos (dec / RAD_TO_DEG) * t * 360 / 86164; // 86164 = sidereal day in secs
		
		// Convert to angle units:
		switch (aUnits)
		{
		case "degrees": aT = trailDegs; break;
		case "radians": aT = trailDegs / RAD_TO_DEG; break;
		case "arcmin": aT = trailDegs * 60; break;
		case "arcsec": aT = trailDegs * 3600; break;
		}
		
		var aToverW = aT / aW;
		var aToverH = aT / aH;
		el_res.innerHTML = "Trail length = " + aT.toPrecision (3) + " " + aUnits + 
		"<br/> = " + aToverW.toPrecision (3) + " of field width" +
		"<br/> = " + aToverH.toPrecision (3) + " of field height";

		var px = Math.max (wd, ht) * Math.max (aToverW, aToverH);
		el_resim.innerHTML = "Trail length = " + px.toPrecision (3) + " pixels";
	}
}

function getPositiveFloatField (field, labelId)
{
	var x = parseFloat (field);
	var el = document.getElementById (labelId);
	if (isNaN (x) || x <= 0) { el.style.color = "#f00"; foundErrors = true; } else el.style.color = "#555";
	return x;
}

function getPositiveFloatFieldWithLimits (field, labelId, min, max)
{
	var x = parseFloat (field);
	var el = document.getElementById (labelId);
	if (isNaN (x) || x < min || x > max) { el.style.color = "#f00"; foundErrors = true; } else el.style.color = "#555";
	return x;
}
